@@ -11,6 +11,7 @@
  * File.
  *
  * For license terms, see the file COPYING.
+ * SPDX-License-Identifier: BSD-2-Clause
  */
 
 #ifndef S_SPLINT_S
@@ -33,9 +34,9 @@ static bool terse = false;
 static bool help = false;
 static bool line = false;
 
-enum { decimal, hex, octal } mode;
+enum { decimal, hex, octal, binary } mode;
 
-void print_table(unsigned short delivery);
+void print_table(unsigned short delivery, bool vertical);
 
 static string cnames[128][16] =
 {
@@ -65,7 +66,7 @@ char	*str;
 void 
 showHelp(FILE *out, char *progname) 
 {
-  fprintf(out,"Usage: %s [-dxohv] [-t] [char-alias...]\n", progname); 
+  fprintf(out,"Usage: %s [-adxohv] [-t] [char-alias...]\n", progname); 
 #define P(s)	(void)fputs(s "\n", out);
 #include "splashscreen.h"
 #undef P
@@ -76,10 +77,13 @@ showHelp(FILE *out, char *progname)
 static void process_options(int argc, char *argv[], char * opt_string) 
 {
     int op;
+    bool vertical = false;
 
     while( (op=getopt(argc, argv, opt_string)) != -1) {
-
 	switch (op) {
+	case 'a':
+	    vertical = true;
+	    break;
 	case 't':
 	    terse = true; 
 	    break;
@@ -88,13 +92,16 @@ static void process_options(int argc, char *argv[], char * opt_string)
 	    line = true; 
 	    break;
 	case 'd':
-            print_table(decimal);
+            print_table(decimal, vertical);
             exit(0);
 	case 'x':
-            print_table(hex);
+            print_table(hex, vertical);
             exit(0);
 	case 'o':
-            print_table(octal);
+            print_table(octal, vertical);
+            exit(0);
+	case 'b':
+            print_table(binary, vertical);
             exit(0);
 	case '?':
 	case 'h':
@@ -361,25 +368,52 @@ static void ascii(char *str)
     } /* outer if */
 }
 
-void print_table(unsigned short delivery)
+static char *bin(int n)
+{
+    static char rep[9];
+    int c, k;
+
+    rep[0] = '\0';
+    for (c = 6; c >= 0; c--)
+    {
+        k = n >> c;
+
+        if (k & 1)
+            strncat(rep, "1", sizeof(rep));
+        else
+            strncat(rep, "0", sizeof(rep));
+    }
+
+    return rep;
+}
+
+void print_table(unsigned short delivery, bool vertical)
 {
    static unsigned short i, j, len;
    char   separator[]= "   "; 
    char   *tail     = separator + 3 ;
    char   *space; 
-   char   *name ;
+   char   *name;
+   int rows, cols;
 
-   for (i=0;i<16; i++) {
-           for (j=0; j<8; j++) {
-                  name= *cnames[i+(j*16)];
+   if (vertical)
+       cols = 4;
+   else
+       cols = 8;
+   rows = 128 / cols;
+   for (i = 0; i < rows; i++) {
+           for (j = 0; j < cols; j++) {
+                  name= *cnames[i+(j*rows)];
                   len = strlen(name);
-                  space = tail - (len % 3) ; 
+                  space = tail - (len % 3) ;
                   switch (delivery) {
-                      case decimal:  printf("%5d %1s%1s",i+(j*16),name,space);
+                      case decimal:  printf("%5d %1s%1s",i+(j*rows),name,space);
                                      break;
-                      case octal  :  printf("%5o %1s%1s",i+(j*16),name,space);
+                      case octal  :  printf("  %03o %1s%1s",i+(j*rows),name,space);
                                      break;
-                      case hex    :  printf("%5X %1s%1s",i+(j*16),name,space);
+                      case hex    :  printf("   %02X %1s%1s",i+(j*rows),name,space);
+                                     break;
+		      case binary :  printf("   %s %1s%1s",bin(i+(j*rows)),name,space);
                                      break;
                   }
            }
@@ -390,7 +424,7 @@ void print_table(unsigned short delivery)
 
 int main(int argc, char **argv)
 {
-    char      *optstring="tshvxod" ;
+    char      *optstring="abtshvxod" ;
 
 
     process_options(argc, argv, optstring) ;
